# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Clear memory
rm(list = ls())

### Load Apollo library
library(apollo)

### Initialise code
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName  ="Apollo_example_30",
  modelDescr ="Simple MNL model with many alternatives",
  indivID    ="ID"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### For this example, we generate a database with 2000 observations
  # following a logit generation process.
  # No panel data (i.e. each obs is from a different individual).
  # 100 alternatives, each with two attributes: x1 and x2
  # True coefficients are 1 and -1 for x1 and x2, respectively.
set.seed(24)
N  = 2000
J  = 100
X1 = matrix(5*runif(N*J)   , nrow=N, ncol=J, dimnames=list(c(), paste0("x1_",1:J)))
X2 = matrix(5*runif(N*J)   , nrow=N, ncol=J, dimnames=list(c(), paste0("x2_",1:J)))
av = matrix(runif(N*J)>=0.3, nrow=N, ncol=J, dimnames=list(c(), paste0("av" ,1:J)))
ep = -log(-log(matrix(runif(N*J), nrow=N, ncol=J)))
U  = 1*X1 - 1*X2 + ep
U[!av] = -Inf
y  = apply(U, MARGIN=1, which.max)
database = data.frame(ID=1:N, choice=y, X1, X2, av)
rm(N,J,X1,X2,av,ep,U,y)


# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Vector of parameters, including any that are kept fixed in estimation
apollo_beta=c(b1 = 0,
              b2 = 0)

### Vector with names (in quotes) of parameters to be kept fixed at their starting value in apollo_beta, use apollo_beta_fixed = c() if none
apollo_fixed = c()

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){

  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))

  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  J = 100
  V = list()
  for(j in 1:J) V[[paste0("alt",j)]] = b1*get(paste0("x1_",j)) + b2*get(paste0("x2_",j))
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives  = setNames(1:J, names(V)), 
    avail         = setNames(apollo_inputs$database[,paste0("av",1:J)], names(V)), 
    choiceVar     = choice,
    V             = V
  )
  
  ### Compute probabilities using MNL model
  P[['model']] = apollo_mnl(mnl_settings, functionality)

  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)

# ################################################################# #
#### MODEL OUTPUTS                                               ####
# ################################################################# #

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO SCREEN)                               ----
# ----------------------------------------------------------------- #

apollo_modelOutput(model)

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO FILE, using model name)               ----
# ----------------------------------------------------------------- #

apollo_saveOutput(model)


